@@ -101,16 +101,21 @@
 {
     if (is.null(x))
         return(NULL)
-    if (any(.isStructural(unlist(x, use.names=FALSE))))
+    flat <- unlist(x, use.names=FALSE)
+    if (any(.isStructural(flat))) {
         CharacterList(x)
-    else DNAStringSetList(x)
+    } else {
+        flat[grepl("I", flat, fixed=TRUE)] <- "."
+        flat[grepl("*", flat, fixed=TRUE)] <- "" 
+        DNAStringSetList(relist(flat, x))
+    }
 }
 
 .isStructural <- function(x)
 {
     grepl("<", x, fixed=TRUE) |
     grepl("[", x, fixed=TRUE) |
-    grepl("]", x, fixed=TRUE)
+    grepl("]", x, fixed=TRUE) 
 }
 
 .formatInfo <- function(x, hdr, nrecords)
@@ -159,7 +164,7 @@
 }
 
 .pasteCollapseRows <- function(x, sep = ",") {
-  if (!is.matrix(x) || mode(x) != "character") {
+  if (!is.matrix(x) || storage.mode(x) != "character") {
     stop("'x' must be a matrix of mode character")
   }
   if (!isSingleString(sep) || nchar(sep) == 0L) {
@@ -198,7 +203,7 @@
     if (missing(keys))
         return(FALSE)
     if (any(mkeys <- !keys %in% keys(x))) {
-        msg <- paste(BiocGenerics:::selectSome(keys[mkeys]), collapse=" ")
+        msg <- paste(S4Vectors:::selectSome(keys[mkeys]), collapse=" ")
         warning(sum(mkeys), " keys not found in ", db, " database: ", msg,
                 call.=FALSE)
     }
@@ -210,7 +215,7 @@
     if (missing(cols))
         return(FALSE)
     if (any(mcols <- !cols %in% columns(x))) {
-        msg <- paste(BiocGenerics:::selectSome(cols[mcols], 5), collapse=" ")
+        msg <- paste(S4Vectors:::selectSome(cols[mcols], 5), collapse=" ")
         warning(sum(mcols), " columns not found in ", db, " database: ", msg,
                 call.=FALSE)
         return(TRUE)
@@ -303,4 +308,3 @@
                   TXID=txid, CDSID=cdsid))
     res 
 }
-
