@@ -9,7 +9,7 @@
 
 VCF <-
     function(rowRanges=GRanges(), colData=DataFrame(), 
-             exptData=SimpleList(header=VCFHeader()), 
+             exptData=list(header=VCFHeader()), 
              fixed=DataFrame(),
              info=DataFrame(row.names=seq_along(rowRanges)), 
              geno=SimpleList(),
@@ -37,7 +37,8 @@ VCF <-
     }
 
     new(class, SummarizedExperiment(assays=geno, rowRanges=rowRanges,
-        colData=colData, exptData=exptData), fixed=fixed, info=info, ...)
+        colData=colData, metadata=as.list(exptData)), fixed=fixed, info=info,
+        ...)
 }
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -46,7 +47,7 @@ VCF <-
 
 .checkLength <- function (x, len) 
 {
-    if (len != length(slot(x, "rowData"))) 
+    if (len != length(slot(x, "rowRanges"))) 
         stop("length(value) must equal length(rowRanges(x))")
 }
 
@@ -137,7 +138,7 @@ setReplaceMethod("fixed", c("VCF", "DataFrame"),
 setMethod("rowRanges", "VCF", 
     function(x, ..., fixed = TRUE) 
 {
-    gr <- slot(x, "rowData")
+    gr <- slot(x, "rowRanges")
     if (fixed)
         if (length(slot(x, "fixed")) != 0L)
             mcols(gr) <- append(mcols(gr), slot(x, "fixed"))
@@ -149,13 +150,13 @@ setReplaceMethod("rowRanges", c("VCF", "GRanges"),
 {
     fixed_idx <- names(mcols(value)) %in% c("REF", "ALT", "QUAL", "FILTER")
     slot(x, "fixed") <- mcols(value)[fixed_idx] 
-    slot(x, "rowData") <- value[,!fixed_idx]
+    slot(x, "rowRanges") <- value[,!fixed_idx]
     validObject(x)
     x
 })
 
 ### Must define VCF methods for 'mcols<-' and 'dimnames<-' instead of 
-### inheriting from SummarizedExperiment. 'fixed' fields are stored in 
+### inheriting from RangedSummarizedExperiment. 'fixed' fields are stored in 
 ### a separate slot and not as metadata columns of 'rowRanges'.
 
 setReplaceMethod("mcols", c("VCF", "DataFrame"),
@@ -163,18 +164,18 @@ setReplaceMethod("mcols", c("VCF", "DataFrame"),
 {
     idx <- names(value) %in% "paramRangeID"
     fixed(x) <- value[!idx] 
-    slot(x, "rowData") <- value[,idx]
+    slot(x, "rowRanges") <- value[,idx]
     x
 })
 
 setReplaceMethod("dimnames", c("VCF", "list"),
     function(x, value)
 {
-    rowRanges <- slot(x,"rowData")
+    rowRanges <- slot(x, "rowRanges")
     names(rowRanges) <- value[[1]]
     colData <- colData(x)
     rownames(colData) <- value[[2]]
-    GenomicRanges:::clone(x, rowData=rowRanges, colData=colData)
+    BiocGenerics:::replaceSlots(x, rowRanges=rowRanges, colData=colData)
 })
  
 ### info 
@@ -269,13 +270,13 @@ setReplaceMethod("strand", "VCF",
 setMethod("header", "VCF",
     function(x)
 { 
-    exptData(x)$header
+    metadata(x)$header
 })
 
 setReplaceMethod("header", c("VCF", "VCFHeader"),
     function(x, value)
 {
-    slot(x, "exptData")$header <- value
+    slot(x, "metadata")$header <- value
     .valid.VCFHeadervsVCF(x)
     x
 })
@@ -292,28 +293,21 @@ setMethod("[", c("VCF", "ANY", "ANY"),
 
     if (!missing(i) && is.character(i)) {
         msg <- "<VCF>[i,] index out of bounds: %s"
-        i <- GenomicRanges:::.SummarizedExperiment.charbound(i, rownames(x), msg)
+        i <- SummarizedExperiment:::.SummarizedExperiment.charbound(i, rownames(x), msg)
     }
 
-    ii <- ff <- NULL
-    if (!missing(i)) {
-        if (length(slot(x, "info")) != 0L)
-            ii <- i
-        if (length(slot(x, "fixed")) != 0L)
-            ff <- i
-    }
     if (missing(i) && missing(j)) {
         x
     } else if (missing(i)) {
         callNextMethod(x, , j, ...)
     } else if (missing(j)) {
-        callNextMethod(x, i, , rowData=slot(x, "rowData")[i,,drop=FALSE],
-                       info=slot(x, "info")[ii,,drop=FALSE],
-                       fixed=slot(x, "fixed")[ff,,drop=FALSE], ...)
+        callNextMethod(x, i, ,
+                       info=slot(x, "info")[i,,drop=FALSE],
+                       fixed=slot(x, "fixed")[i,,drop=FALSE], ...)
     } else {
-        callNextMethod(x, i, j, rowData=slot(x, "rowData")[i,,drop=FALSE],
-                       info=slot(x, "info")[ii,,drop=FALSE],
-                       fixed=slot(x, "fixed")[ff,,drop=FALSE], ...)
+        callNextMethod(x, i, j,
+                       info=slot(x, "info")[i,,drop=FALSE],
+                       fixed=slot(x, "fixed")[i,,drop=FALSE], ...)
     }
 })
 
@@ -323,7 +317,7 @@ setReplaceMethod("[",
 {
     if (!missing(i) && is.character(i)) {
         msg <- "<VCF>[i,]<- index out of bounds: %s"
-        i <- GenomicRanges:::.SummarizedExperiment.charbound(i, rownames(x), msg)
+        i <- SummarizedExperiment:::.SummarizedExperiment.charbound(i, rownames(x), msg)
     }
 
     if (missing(i) && missing(j)) {
@@ -331,12 +325,8 @@ setReplaceMethod("[",
     } else if (missing(i)) {
         callNextMethod(x, , j, ..., value=value)
     } else if (missing(j)) {
-        callNextMethod(x, i, , rowData=local({
-            rd <- slot(x, "rowData")
-            rd[i,] <- slot(value, "rowData")
-            names(rd)[i] <- names(slot(value, "rowData"))
-            rd
-        }), info=local({
+        callNextMethod(x, i, ,
+            info=local({
             ii <- slot(x, "info")
             ii[i,] <- slot(value, "info")
             ii 
@@ -346,12 +336,8 @@ setReplaceMethod("[",
             ff
         }), ..., value=value)
     } else {
-        callNextMethod(x, i, j, rowData=local({
-            rd <- slot(x, "rowData")
-            rd[i,] <- slot(value, "rowData")
-            names(rd)[i] <- names(slot(value, "rowData"))
-            rd
-        }), info=local({
+        callNextMethod(x, i, j,
+            info=local({
             ii <- slot(x, "info")
             ii[i,] <- slot(value, "info")
             ii 
@@ -377,15 +363,18 @@ setMethod("rbind", "VCF",
     if (!.compare(lapply(args, ncol)))
             stop("'...' objects must have the same number of samples")
 
-    rowRanges <- do.call(c, lapply(args,
-        function(i) slot(i, "rowData")))
+    fixed <- do.call(rbind, lapply(args, fixed))
+    info <- do.call(rbind, lapply(args, info))
+    rowRanges <- do.call(c, lapply(args, rowRanges))
     colData <- GenomicRanges:::.cbind.DataFrame(args, colData, "colData")
     assays <- GenomicRanges:::.bind.arrays(args, rbind, "assays")
-    exptData <- do.call(c, lapply(args, exptData))
-    info <- do.call(rbind, lapply(args, info))
-    fixed <- do.call(rbind, lapply(args, fixed))
-    new(class(args[[1]]), assays=assays, rowData=rowRanges,
-        colData=colData, exptData=exptData, fixed=fixed, info=info)
+    elementMetadata <- do.call(rbind, lapply(args, slot, "elementMetadata"))
+    metadata <- do.call(c, lapply(args, metadata))
+
+    BiocGenerics:::replaceSlots(args[[1L]],
+        fixed=fixed, info=info,
+        rowRanges=rowRanges, colData=colData, assays=assays,
+        elementMetadata=elementMetadata, metadata=metadata)
 })
 
 .renameSamples <- function(args)
@@ -405,24 +394,23 @@ setMethod("cbind", "VCF",
     args <- unname(list(...))
     if (!.compare(lapply(args, class)))
         stop("'...' objects must be of the same VCF class")
-
-
     if (!.compare(lapply(args, rowRanges), TRUE))
         stop("'...' object ranges (rows) are not compatible")
-    rowRanges <- rowRanges(args[[1]])
-    mcols(rowRanges) <- GenomicRanges:::.cbind.DataFrame(args, mcols, "mcols")
-    info <- GenomicRanges:::.cbind.DataFrame(args, info, "info") 
-    colData <- do.call(rbind, lapply(args, colData))
-    assays <- GenomicRanges:::.bind.arrays(args, cbind, "assays")
-    exptData <- do.call(c, lapply(args, exptData))
     if (!.compare(lapply(args, "fixed")))
         stop("data in 'fixed(VCF)' must match.")
-    else
-        fixed <- fixed(args[[1]]) 
-    new(class(args[[1]]), assays=assays, rowData=rowRanges,
-        colData=colData, exptData=exptData, fixed=fixed, info=info)
 
+    fixed <- fixed(args[[1L]])
+    info <- GenomicRanges:::.cbind.DataFrame(args, info, "info") 
+    rowRanges <- rowRanges(args[[1L]])
+    mcols(rowRanges) <- GenomicRanges:::.cbind.DataFrame(args, mcols, "mcols")
+    colData <- do.call(rbind, lapply(args, colData))
+    assays <- GenomicRanges:::.bind.arrays(args, cbind, "assays")
+    metadata <- do.call(c, lapply(args, metadata))
 
+    BiocGenerics:::replaceSlots(args[[1L]],
+        fixed=fixed, info=info,
+        rowRanges=rowRanges, colData=colData, assays=assays,
+        metadata=metadata)
 })
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -519,21 +507,6 @@ setMethod(show, "VCF",
     }
 }
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### updateObject 
-###
-
-### Creates a CollapsedVCF from a VCF.
-setMethod("updateObject", "VCF",
-    function(object, ..., verbose=FALSE)
-    {
-        if (verbose) 
-            message("updateObject(object = 'VCF')") 
-        VCF(rowRanges=rowRanges(object), colData=colData(object), exptData=exptData(object), 
-            info=mcols(info(object))[-1], fixed=mcols(fixed(object))[-1], geno=geno(object))
-    }
-)
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### restrictToSNV 
 ###
