@@ -111,7 +111,7 @@ VRanges <-
            hardFilters = FilterRules())
 {
   gr <- GRanges(seqnames, ranges,
-                strand = .rleRecycleVector("+", length(ranges)), ...)
+                strand = .rleRecycleVector("*", length(ranges)), ...)
   if (length(gr) == 0L && length(ref) == 0L) {
     maxLen <- 0L
   } else {
@@ -129,7 +129,7 @@ VRanges <-
   altDepth <- .rleRecycleVector(altDepth, maxLen)
   totalDepth <- .rleRecycleVector(totalDepth, maxLen)
   softFilterMatrix <- as.matrix(softFilterMatrix)
-  mode(softFilterMatrix) <- "logical"
+  storage.mode(softFilterMatrix) <- "logical"
   softFilterMatrix.ind <-
     S4Vectors:::recycleVector(seq_len(nrow(softFilterMatrix)), maxLen)
   softFilterMatrix <- softFilterMatrix[softFilterMatrix.ind,,drop=FALSE]
@@ -409,7 +409,7 @@ vranges2Vcf <- function(x, info = character(), filter = character(),
                         FORMAT = makeFORMATheader(mcols(x)[genoMCols]),
                         INFO = makeINFOheader(mcols(xUniq)[info]),
                         FILTER = makeFILTERheader(x)))
-  exptData <- SimpleList(header = header)
+  metadata <- list(header = header)
 
   alt <- as.character(alt(xUniq))
   qual <- xUniq$QUAL
@@ -475,7 +475,7 @@ vranges2Vcf <- function(x, info = character(), filter = character(),
 
   info <- mcols(xUniq)[info]
   
-  VCF(rowRanges = rowRanges, colData = colData, exptData = exptData,
+  VCF(rowRanges = rowRanges, colData = colData, metadata = metadata,
       fixed = fixed, geno = geno, info = info, collapsed = FALSE)
 }
 
@@ -650,7 +650,7 @@ pileupGRanges <- function(x) {
 ###
 
 .showHardFilters <- function(object) {
-  cat(BiocGenerics:::labeledLine("  hardFilters", names(hardFilters(object))))
+  cat(S4Vectors:::labeledLine("  hardFilters", names(hardFilters(object))))
 }
 
 setMethod("show", "VRanges", function(object) {
