@@ -2,17 +2,17 @@
 ### predictCoding methods 
 ### =========================================================================
 
-setMethod("predictCoding", c("Ranges", "TranscriptDb", "ANY", "DNAStringSet"),
+setMethod("predictCoding", c("Ranges", "TxDb", "ANY", "DNAStringSet"),
     function(query, subject, seqSource, varAllele, ..., ignore.strand=FALSE)
 {
     callGeneric(as(query, "GRanges"), subject, seqSource, varAllele, ...,
                 ignore.strand=ignore.strand) 
 })
 
-setMethod("predictCoding", c("CollapsedVCF", "TranscriptDb", "ANY", "missing"),
+setMethod("predictCoding", c("CollapsedVCF", "TxDb", "ANY", "missing"),
     function(query, subject, seqSource, varAllele, ..., ignore.strand=FALSE)
 {
-    rd <- rowData(query)
+    rd <- rowRanges(query)
     alt <- alt(query) 
     if (is(alt, "CharacterList")) {
         alt <- .toDNAStringSetList(alt)
@@ -20,30 +20,49 @@ setMethod("predictCoding", c("CollapsedVCF", "TranscriptDb", "ANY", "missing"),
             stop("No nucleotide ALT values were detected.")
         }
     }
-    rd <- rep(rowData(query), elementLengths(alt))
+    rd <- rep(rowRanges(query), elementLengths(alt))
     res <- callGeneric(rd, subject, seqSource, unlist(alt, use.names=FALSE), 
                 ..., ignore.strand=ignore.strand)
-    ## adjust QUERYID for expansion of rowData
+    ## adjust QUERYID for expansion of rowRanges
     res$QUERYID <- rep(seq_len(length(alt)),
                        elementLengths(alt))[res$QUERYID]
     res 
 })
 
-setMethod("predictCoding", c("ExpandedVCF", "TranscriptDb", "ANY", "missing"),
+setMethod("predictCoding", c("ExpandedVCF", "TxDb", "ANY", "missing"),
     function(query, subject, seqSource, varAllele, ..., ignore.strand=FALSE)
 {
-    ## ExpandedVCF ALT must be DNAStringSet (CharacterList not supported)
-    callGeneric(rowData(query), subject, seqSource, alt(query), ..., 
+    if (is(alt(query), "CharacterList")) {
+      stop("alt(query) must be a DNAStringSet (not a CharacterList)")
+    }
+    callGeneric(rowRanges(query), subject, seqSource, alt(query), ..., 
                 ignore.strand=ignore.strand) 
 })
 
-setMethod("predictCoding", c("GRanges", "TranscriptDb", "ANY", "DNAStringSet"),
+setMethod("predictCoding", c("GRanges", "TxDb", "ANY", "DNAStringSet"),
     function(query, subject, seqSource, varAllele, ..., ignore.strand=FALSE)
 {
     .predictCoding(query, subject, seqSource, varAllele, ...,
                    ignore.strand=ignore.strand)
 })
 
+setMethod("predictCoding", c("VRanges", "TxDb", "ANY", "missing"),
+    function(query, subject, seqSource, varAllele, ..., ignore.strand=FALSE)
+{
+    varAllele <- alt(query)
+    query <- as(query, "GRanges")
+    if (!is(varAllele, "DNAStringSet")) {
+        tryCatch({
+            varAllele <- DNAStringSet(varAllele)
+        }, error=function(e) {
+            stop(paste0("attempt to coerce 'alt' to DNAStringSet failed with ",
+                 "error: ", conditionMessage(e)))
+        })
+    }
+    .predictCoding(query, subject, seqSource, varAllele, ...,
+                   ignore.strand=ignore.strand)
+})
+
 .predictCoding <-
     function(query, subject, seqSource, varAllele, ..., 
              cache=new.env(parent=emptyenv()), ignore.strand=FALSE)
@@ -60,14 +79,14 @@ setMethod("predictCoding", c("GRanges", "TranscriptDb", "ANY", "DNAStringSet"),
 
     map <- data.frame(geneid=rep(names(cache[["txbygene"]]), 
                           elementLengths(cache[["txbygene"]])),
-                      txid=values(unlist(cache[["txbygene"]], 
+                      txid=mcols(unlist(cache[["txbygene"]], 
                           use.names=FALSE))[["tx_id"]],
                       stringsAsFactors=FALSE)
 
     txlocal <- .predictCodingGRangesList(query, cache[["cdsbytx"]], 
                    seqSource, varAllele, ..., ignore.strand=ignore.strand)
-    txid <- values(txlocal)[["TXID"]] 
-    values(txlocal)[["GENEID"]] <- map$geneid[match(txid, map$txid)]
+    txid <- mcols(txlocal)$TXID 
+    mcols(txlocal)$GENEID <- map$geneid[match(txid, map$txid)]
     txlocal
 }
 
@@ -76,61 +95,67 @@ setMethod("predictCoding", c("GRanges", "TranscriptDb", "ANY", "DNAStringSet"),
                                       if.fuzzy.codon="error", 
                                       ignore.strand=FALSE)
 {
-    ## FIXME : set query back after olaps
-    if (any(insertion <- width(query) == 0))
-        start(query)[insertion] <- start(query)[insertion] - 1
     if (ignore.strand)
         strand(query) <- "*"
 
-    ## retrieve local coordinates
-    values(query) <- append(values(query), DataFrame(varAllele=varAllele))
-    txlocal <- refLocsToLocalLocs(ranges=query, cdsbytx=cdsbytx)
-
+    ## variant location in cds region
+    mcols(query) <- append(mcols(query), DataFrame(varAllele=varAllele))
+    txlocal <- .localCoordinates(query, cdsbytx, ignore.strand=FALSE, ...)
     if (length(txlocal) == 0)
         return(txlocal)
- 
-    rwidth <- width(txlocal)
-    translateidx <- rep(TRUE, length(txlocal)) 
-    ## reverse complement variant alleles for "-" strand
+
+    ## reverse complement "-" strand
+    valid <- rep(TRUE, length(txlocal))
     nstrand <- as.vector(strand(txlocal) == "-")
-    if (any(nstrand))
-        values(txlocal)[["varAllele"]][nstrand] <-
-            reverseComplement(values(txlocal)[["varAllele"]][translateidx & nstrand])
-    altallele <- values(txlocal)[["varAllele"]]
-    fmshift <- abs(width(altallele) - rwidth) %% 3 != 0 
+    if (any(nstrand)) {
+        va <- mcols(txlocal)$varAllele
+        va[nstrand] <- reverseComplement(va[valid & nstrand])
+        mcols(txlocal)$varAllele <- va
+    }
+
+    ## frameshift
+    refwidth <- width(txlocal)
+    altallele <- mcols(txlocal)$varAllele
+    fmshift <- abs(width(altallele) - refwidth) %% 3 != 0 
     if (any(fmshift))
-        translateidx[fmshift] <- FALSE
+        valid[fmshift] <- FALSE
+
+    ## zero-width
     zwidth <- width(altallele) == 0
     if (any(zwidth)) {
         warning("records with missing 'varAllele' were ignored")
-        translateidx[zwidth] <- FALSE 
+        valid[zwidth] <- FALSE 
         fmshift[zwidth] <- FALSE
     }
-    codeN <- rep(FALSE, length(txlocal)) 
-    codeN[grep("N", as.character(altallele, use.names=FALSE), 
-        fixed=TRUE)] <-TRUE
-    if (any(codeN)) {
-        warning("varAllele values containing 'N' were not translated")
-        translateidx[codeN] <- FALSE
-    }
 
-    ## reference and variant codon sequences
-    altpos <- (start(values(txlocal)[["CDSLOC"]]) - 1L) %% 3L + 1L
-    refCodon <- varCodon <- .constructRefSequences(txlocal, altpos, seqSource, 
-                                                   cdsbytx)
-    if (any(translateidx)) {
-        subseq(varCodon, altpos, width=rwidth)[translateidx]  <-
-            altallele[translateidx]
+    ## reference codon sequences
+    altpos <- (start(mcols(txlocal)$CDSLOC) - 1L) %% 3L + 1L
+    refCodon <- varCodon <- .getRefCodons(txlocal, altpos, seqSource, cdsbytx)
 
-        ## translation
-        refAA <- translate(refCodon, genetic.code=genetic.code,
-                           if.fuzzy.codon=if.fuzzy.codon)
+    ## allowed characters that can't be translated
+    ## "N", ".", "+" and "-"
+    pattern <- "N|\\.|\\+|\\-"
+    altCheck <- grepl(pattern, as.character(altallele, use.names=FALSE))
+    refCheck <- grepl(pattern, as.character(refCodon, use.names=FALSE))
+    noTrans <- rep(FALSE, length(txlocal)) 
+    noTrans[altCheck | refCheck] <- TRUE
+    valid[noTrans] <- FALSE
+    if (any(altCheck))
+        warning("'varAllele' values with 'N', '.', '+' or '-'",
+                " were not translated")
+    if (any(refCheck))
+        warning("reference codons with 'N', '.', '+' or '-'",
+                " were not translated")
+
+    ## substitute and translate
+    refAA <- varAA <- AAStringSet(rep("", length(txlocal))) 
+    if (any(valid)) {
+        subseq(varCodon, altpos, width=refwidth) <- altallele
+        refAA[valid] <- translate(refCodon[valid], genetic.code=genetic.code,
+                                  if.fuzzy.codon=if.fuzzy.codon)
         varAA <- AAStringSet(rep("", length(txlocal))) 
-        varAA[translateidx] <- translate(varCodon[translateidx],
-                                         genetic.code=genetic.code, 
-                                         if.fuzzy.codon=if.fuzzy.codon)
-    } else {
-        refAA <- varAA <- AAStringSet(rep("", length(txlocal))) 
+        varAA[valid] <- translate(varCodon[valid], genetic.code=genetic.code, 
+                                  if.fuzzy.codon=if.fuzzy.codon)
     }
 
     ## results
@@ -139,10 +164,10 @@ setMethod("predictCoding", c("GRanges", "TranscriptDb", "ANY", "DNAStringSet"),
     consequence[nonsynonymous] <- "nonsynonymous" 
     consequence[fmshift] <- "frameshift"
     consequence[nonsynonymous & (as.character(varAA) %in% "*")] <- "nonsense" 
-    consequence[zwidth | codeN] <- "not translated" 
+    consequence[zwidth | noTrans] <- "not translated" 
     consequence <- factor(consequence) 
  
-    values(txlocal) <- append(values(txlocal), 
+    mcols(txlocal) <- append(mcols(txlocal), 
         DataFrame(GENEID=NA_character_, 
                   CONSEQUENCE=consequence, 
                   REFCODON=refCodon, 
@@ -151,14 +176,14 @@ setMethod("predictCoding", c("GRanges", "TranscriptDb", "ANY", "DNAStringSet"),
     txlocal 
 }
 
-.constructRefSequences <- function(txlocal, altpos, seqSource, cdsbytx)
+.getRefCodons <- function(txlocal, altpos, seqSource, cdsbytx)
 { 
     ## adjust codon end for 
     ## - width of the reference sequence
     ## - position of alt allele substitution in the codon
-    cstart <- ((start(values(txlocal)[["CDSLOC"]]) - 1L) %/% 3L) * 3L + 1L
+    cstart <- ((start(mcols(txlocal)$CDSLOC) - 1L) %/% 3L) * 3L + 1L
     cend <- cstart + (((altpos + width(txlocal) - 2L) %/% 3L) * 3L + 2L)
-    txord <- match(values(txlocal)[["TXID"]], names(cdsbytx))
-    txseqs <- getTranscriptSeqs(cdsbytx[txord], seqSource)
+    txord <- match(mcols(txlocal)$TXID, names(cdsbytx))
+    txseqs <- extractTranscriptSeqs(seqSource, cdsbytx[txord])
     DNAStringSet(substring(txseqs, cstart, cend))
 }
