@@ -104,7 +104,7 @@ setMethod(readVcf, c(file="character", genome="missing",
         if (any(!chr %in% seqnames(genome)))
             stop("'seqnames' in 'vcfWhich(param)' must be present in 'genome'")
     }
-    .scanVcfToVCF(scanVcf(file, param=param, row.names=row.names), file, 
+    .scanVcfToVCF(scanVcf(file, param=param, row.names=row.names, ...), file, 
                   genome, param)
 }
 
@@ -154,7 +154,7 @@ setMethod(readVcf, c(file="character", genome="missing",
     geno <- SimpleList(lapply(vcf$GENO, `dimnames<-`, NULL))
 
     vcf <- NULL
-    VCF(rowRanges=rowRanges, colData=colData, exptData=SimpleList(header=hdr),
+    VCF(rowRanges=rowRanges, colData=colData, exptData=list(header=hdr),
         fixed=fixed, info=info, geno=geno)
 }
 
