@@ -10,7 +10,7 @@
 #include "samtools/khash.h"
 #include "strhash.h"
 
-enum { ROWDATA_IDX = 0, REF_IDX, ALT_IDX, QUAL_IDX, FILTER_IDX,
+enum { ROWRANGES_IDX = 0, REF_IDX, ALT_IDX, QUAL_IDX, FILTER_IDX,
        INFO_IDX, GENO_IDX };
 enum { POS_IDX = 0, ID_IDX };
 
@@ -139,17 +139,17 @@ static struct vcftype_t *_vcf_alloc(const int vcf_n, SEXP smap,
     SEXP elt;
     SEXPTYPE type;
     const char *n;
-    struct vcftype_t *vcf, *rowData;
+    struct vcftype_t *vcf, *rowRanges;
 
     vcf = _vcftype_new(VECSXP, NILSXP, '\0', NULL, N_FLDS, 1, 1, 0);
 
     /* FIXED fields */
-    rowData = _vcftype_new(VECSXP, VECSXP, '\0', NULL, 2, 1, 1, 0);
-    rowData->u.list[POS_IDX] =
+    rowRanges = _vcftype_new(VECSXP, VECSXP, '\0', NULL, 2, 1, 1, 0);
+    rowRanges->u.list[POS_IDX] =
         _vcftype_new(INTSXP, NILSXP, '\0', NULL, vcf_n, 1, 1, 0);
-    rowData->u.list[ID_IDX] =
+    rowRanges->u.list[ID_IDX] =
         _vcftype_new(STRSXP, NILSXP, '\0', NULL, vcf_n, 1, 1, 0);
-    vcf->u.list[ROWDATA_IDX] = rowData;
+    vcf->u.list[ROWRANGES_IDX] = rowRanges;
 
     const char *blank = _strhash_put(str, ""), *dot = _strhash_put(str, ".");
     for (int i = 2; i < Rf_length(fmap); ++i) {
@@ -170,7 +170,7 @@ static struct vcftype_t *_vcf_alloc(const int vcf_n, SEXP smap,
             Rf_error("(internal) unknown 'fixed' field '%s'", nm);
     }
 
-    /* info, geno */
+    /* INFO, FORMAT fields */
     int nonzero = 0;
     for (int i = 0; i < Rf_length(smap); ++i)
         if (INTEGER(smap)[i] != 0)
@@ -185,7 +185,7 @@ static void _vcf_grow(struct vcftype_t * vcf, int vcf_n)
 {
     struct vcftype_t *elt;
 
-    elt = vcf->u.list[ROWDATA_IDX];
+    elt = vcf->u.list[ROWRANGES_IDX];
     for (int i = POS_IDX; i <= ID_IDX; ++i)
         elt->u.list[i] = _vcftype_grow(elt->u.list[i], vcf_n);
 
@@ -196,11 +196,11 @@ static void _vcf_grow(struct vcftype_t * vcf, int vcf_n)
     _types_grow(vcf->u.list[GENO_IDX], vcf_n);
 }
 
-static void _parse(char *line, const int irec,
-                   const struct parse_t *parse)
+static void _parse(char *line, const int irec, 
+                   const struct parse_t *parse, Rboolean row_names)
 {
     khash_t(strhash) *str = parse->str;
-    struct vcftype_t *vcf = parse->vcf, *rowData, *elt, *info;
+    struct vcftype_t *vcf = parse->vcf, *rowRanges, *elt, *info;
     const int imap_n = parse->imap_n, gmap_n = parse->gmap_n;
     const int smap_n = parse->smap_n;
     const char **inms = parse->inms, **gnms = parse->gnms;
@@ -212,16 +212,16 @@ static void _parse(char *line, const int irec,
     struct it_t it0, it1, it2;
     char *sample, *ifld, *ikey;
 
-    /* fixed fields */
+    /* FIXED fields */
     char *chrom, *pos, *id, *ref, *alt, *field;
     int alt_n;
 
     chrom = it_init(&it0, line, '\t'); /* CHROM */
     rle_append(parse->chrom, chrom);
-    rowData = vcf->u.list[ROWDATA_IDX];
+    rowRanges = vcf->u.list[ROWRANGES_IDX];
 
     pos = it_next(&it0);      /* POS */
-    rowData->u.list[POS_IDX]->u.integer[irec] = atoi(pos);
+    rowRanges->u.list[POS_IDX]->u.integer[irec] = atoi(pos);
 
     id = it_next(&it0);       /* ID */
 
@@ -236,18 +236,20 @@ static void _parse(char *line, const int irec,
     _vcftype_set(vcf->u.list[FILTER_IDX], irec,
                  _strhash_put(str, it_next(&it0))); /* FILTER */
 
-    if ('.' == *id && '\0' == *(id + 1)) {
-        /* construct ID if missing:
-           chrom\0pos\0ID\0ref\0alt\0 ==> chrom:pos_ref/alt\0 */
-        *(pos - 1) = ':';
-        *(id - 1) = '_';
-        *(alt - 1) = '/';
-        while (*ref != '\0')
-            *id++ = *ref++;
-        *id = '\0';
-        id = chrom;
+    if (row_names) {
+        if ('.' == *id && '\0' == *(id + 1)) {
+            /* construct ID if missing:
+               chrom\0pos\0ID\0ref\0alt\0 ==> chrom:pos_ref/alt\0 */
+            *(pos - 1) = ':';
+            *(id - 1) = '_';
+            *(alt - 1) = '/';
+            while (*ref != '\0')
+                *id++ = *ref++;
+            *id = '\0';
+            id = chrom;
+        }
+        rowRanges->u.list[ID_IDX]->u.character[irec] = _strhash_put(str, id);
     }
-    rowData->u.list[ID_IDX]->u.character[irec] = _strhash_put(str, id);
 
     /* INFO */
     field = it_next(&it0);
@@ -318,7 +320,8 @@ static void _parse(char *line, const int irec,
     Free(gmapidx);
 }
 
-static SEXP _vcf_as_SEXP(struct parse_t *parse, SEXP fmap, SEXP smap)
+static SEXP _vcf_as_SEXP(struct parse_t *parse, SEXP fmap, SEXP smap,
+                         Rboolean row_names)
 {
     SEXP result = PROTECT(_vcftype_as_SEXP(parse->vcf));
 
@@ -326,12 +329,15 @@ static SEXP _vcf_as_SEXP(struct parse_t *parse, SEXP fmap, SEXP smap)
     SEXP dna = dna_hash_as_DNAStringSet(parse->ref);
     SET_VECTOR_ELT(result, REF_IDX, dna);
 
-    /* rowData: GRanges */
-    SEXP rowData, seqnames, start, width, names;
+    /* rowRanges: GRanges */
+    SEXP rowRanges, seqnames, start, width, names;
     PROTECT(seqnames = rle_as_Rle(parse->chrom));
-    rowData = VECTOR_ELT(result, ROWDATA_IDX);
-    start = VECTOR_ELT(rowData, POS_IDX);
-    names = VECTOR_ELT(rowData, ID_IDX);
+    rowRanges = VECTOR_ELT(result, ROWRANGES_IDX);
+    start = VECTOR_ELT(rowRanges, POS_IDX);
+    if (!row_names)
+        names = R_NilValue;
+    else
+        names = VECTOR_ELT(rowRanges, ID_IDX);
     width = get_XVectorList_width(dna);
 
     SEXP ranges, nmspc, fun, expr;
@@ -340,13 +346,13 @@ static SEXP _vcf_as_SEXP(struct parse_t *parse, SEXP fmap, SEXP smap)
     PROTECT(fun = findFun(install("GRanges"), nmspc));
     PROTECT(expr = lang3(fun, seqnames, ranges));
 
-    SET_VECTOR_ELT(result, ROWDATA_IDX, eval(expr, R_GlobalEnv));
+    SET_VECTOR_ELT(result, ROWRANGES_IDX, eval(expr, R_GlobalEnv));
     UNPROTECT(5);
 
     /* names */
     SEXP nms, sxp = Rf_getAttrib(fmap, R_NamesSymbol), elt;
     PROTECT(nms = Rf_allocVector(STRSXP, Rf_length(result)));
-    SET_STRING_ELT(nms, ROWDATA_IDX, mkChar("rowData"));
+    SET_STRING_ELT(nms, ROWRANGES_IDX, mkChar("rowRanges"));
     SET_STRING_ELT(nms, REF_IDX, mkChar("REF"));
     SET_STRING_ELT(nms, ALT_IDX, mkChar("ALT"));
     SET_STRING_ELT(nms, QUAL_IDX, mkChar("QUAL"));
@@ -474,22 +480,24 @@ static void _parse_grow(struct parse_t *parse, int size)
     parse->vcf_n = size;
 }
 
+/* --- .Call ENTRY POINT --- */
 SEXP scan_vcf_connection(SEXP txt, SEXP smap, SEXP fmap, SEXP imap,
-                         SEXP gmap)
+                         SEXP gmap, SEXP rownames)
 {
     struct parse_t *parse;
+    Rboolean row_names = LOGICAL(rownames)[0];
 
     parse = _parse_new(Rf_length(txt), smap, fmap, imap, gmap);
 
     /* parse each line */
     for (int irec = 0; irec < parse->vcf_n; irec++) {
         char *line = Strdup(CHAR(STRING_ELT(txt, irec)));
-        _parse(line, irec, parse);
+        _parse(line, irec, parse, row_names);
         Free(line);
     }
 
     SEXP result = PROTECT(Rf_allocVector(VECSXP, 1));
-    SET_VECTOR_ELT(result, 0, _vcf_as_SEXP(parse, fmap, smap));
+    SET_VECTOR_ELT(result, 0, _vcf_as_SEXP(parse, fmap, smap, row_names));
     _vcf_types_tidy(parse, result);
     _parse_free(parse);
     UNPROTECT(1);
@@ -497,15 +505,19 @@ SEXP scan_vcf_connection(SEXP txt, SEXP smap, SEXP fmap, SEXP imap,
     return result;
 }
 
-SEXP scan_vcf_character(SEXP file, SEXP yield,
-                        SEXP smap, SEXP fmap, SEXP imap, SEXP gmap)
+/* --- .Call ENTRY POINT --- */
+SEXP scan_vcf_character(SEXP file, SEXP yield, SEXP smap, SEXP fmap, 
+                        SEXP imap, SEXP gmap, SEXP rownames)
 {
     struct parse_t *parse;
+    Rboolean row_names = LOGICAL(rownames)[0];
 
     if (!IS_INTEGER(yield) || 1L != Rf_length(yield))
         Rf_error("'yield' must be integer(1)");
     if (!IS_CHARACTER(file) || 1L != Rf_length(file))
         Rf_error("'file' must be character(1) or as on ?scanVcf");
+    if (!IS_LOGICAL(rownames))
+        Rf_error("'row.names' must be TRUE or FALSE");
 
     parse = _parse_new(INTEGER(yield)[0], smap, fmap, imap, gmap);
 
@@ -545,7 +557,7 @@ SEXP scan_vcf_character(SEXP file, SEXP yield,
             else break;
         }
 
-        _parse(buf0, irec, parse);
+        _parse(buf0, irec, parse, row_names);
 
         irec += 1;
         buf = buf0;
@@ -557,7 +569,7 @@ SEXP scan_vcf_character(SEXP file, SEXP yield,
     _vcf_grow(parse->vcf, irec);
 
     SEXP result = PROTECT(Rf_allocVector(VECSXP, 1));
-    SET_VECTOR_ELT(result, 0, _vcf_as_SEXP(parse, fmap, smap));
+    SET_VECTOR_ELT(result, 0, _vcf_as_SEXP(parse, fmap, smap, row_names));
     _vcf_types_tidy(parse, VECTOR_ELT(result, 0));
     _parse_free(parse);
     UNPROTECT(1);
@@ -565,9 +577,11 @@ SEXP scan_vcf_character(SEXP file, SEXP yield,
     return result;
 }
 
+/* --- .Call ENTRY POINT --- */
 SEXP tabix_as_vcf(tabix_t *tabix, ti_iter_t iter, const int yield,
-                  SEXP state)
+                  SEXP state, SEXP rownames)
 {
+    Rboolean row_names = LOGICAL(rownames)[0];
     const ti_conf_t *conf = ti_get_conf(tabix->idx);
     SEXP sample = VECTOR_ELT(state, 0), fmap = VECTOR_ELT(state, 1);
     const int nrec = NA_INTEGER == yield ? TBX_INIT_SIZE : yield;
@@ -598,7 +612,7 @@ SEXP tabix_as_vcf(tabix_t *tabix, ti_iter_t iter, const int yield,
         memcpy(buf, line, linelen);
         buf[linelen] = '\0';
 
-        _parse(buf, irec, parse);
+        _parse(buf, irec, parse, row_names);
         irec += 1;
         if (NA_INTEGER != yield && irec == parse->vcf_n)
             break;
@@ -614,7 +628,7 @@ SEXP tabix_as_vcf(tabix_t *tabix, ti_iter_t iter, const int yield,
 
     _vcf_grow(parse->vcf, irec);
 
-    SEXP result = PROTECT(_vcf_as_SEXP(parse, fmap, sample));
+    SEXP result = PROTECT(_vcf_as_SEXP(parse, fmap, sample, row_names));
     _vcf_types_tidy(parse, result);
     _parse_free(parse);
     UNPROTECT(1);
